/*
** UC: 21111 - Sistemas Operativos
** e-fólio A 2022-23 (gsn.c)
**
** Aluno: 2104846 - José Barroso
*/
#include <stdio.h> //entrada/saida no terminal
#include <stdlib.h> //para funcao atoi()
#include <unistd.h> //para funcoes do Sistema Operacional como a fork()
#include <sys/wait.h> //para waitpid()


int main(int argc, char *argv[]){
    int n;
    pid_t pidB, pidC, pidD;

    //o terminal vai receber dois comandos: "./gsn" e "n";
    //verificar se o numero de argumentos dado na linha de comando é correto
    if(argc != 2) {
        printf("Erro, numero de argumentos incorreto.\n");
        exit(1); //sair do programa  
    }
     //argv[] é o vetor de strings que recebe os argumentos passados na linha de comando
     //Armazenado em argv[0] é o nome do programa "gsn.c", por isso argv[1] é o argumento "n"
    //funcao atoi() converte o argumento para um valor inteiro, para assim poder comparar se "n<=0"
    n = atoi(argv[1]);
    if(n <= 0) {
        printf("\n Erro n<=0!\n\n"); //para o argumento ser correto, n tem que ser maior que 0 (n>0)
        exit(1); }

    //gsn.c é o Processo A, e o pai do Processo A é a bash
    printf("Processo A: PID=%d PPID=%d\n", getpid(), getppid());
    
    //Passo 1: "head -c NNN /dev/urandom >tmp.bin"
    pidB = fork();  //duplicar o Processo A, criando o processo B

    if(pidB < 0 || pidB==-1) {
        //O perror() é uma função que imprime uma mensagem de erro para a saída padrão de erro (stderr), que por padrão é a tela do terminal.
       perror("Erro na criação do Processo B");
       return 1;
    }
    else if(pidB == 0){
        printf("Processo B: PID=%d PPID=%d\n", getpid(), getppid());
        //antes da substituicao da imagem, usar a funcao freopen() para redirecionar a saida padrao stdout para o ficheiro "tmp.bin"
        freopen("tmp.bin", "w", stdout); 
        execl("/usr/bin/head", "head", "-c", argv[1], "/dev/urandom", NULL);
        exit(1); //encerrar o processo filho B e liberar os seus recursos
}
    waitpid(pidB, NULL, 0); //O Processo A (gsn.c) chama esta funcao para aguardar pela terminacao do processo filho, antes de ser criado o Processo B

    //Passo 2: "hexdump -v -e "/1 \"%d\\n\"" <tmp.bin >tmp.txt"
    pidC = fork();

    if(pidC < 0 || pidC==-1){
        perror("Erro na criacao do Processo C");
        return 1;
    }
    else if(pidC == 0){
        printf("Processo C: PID=%d PPID=%d\n", getpid(), getppid());
        freopen("tmp.bin", "r", stdin);
        freopen("tmp.txt", "w", stdout);
        char* args[] = {"hexdump", "-v", "-e", "/1 \"%d\\n\"", NULL};
        execvp(args[0], args);
        exit(1); //encerrar o processo filho C e liberar os seus recursos
     }

    waitpid(pidC, NULL, 0);

    //Passo 3: "sort -g tmp.txt"
    pidD = fork();
    if(pidD < 0 || pidD==-1){
        perror("Erro na criação do processo D");
        return 1;
    }
    else if (pidD == 0){
        printf("Processo D: PID=%d PPID=%d\n", getpid(), getppid());
        freopen("tmp.txt", "r", stdin);
        char* args[] = {"sort", "-g", NULL};
        execv("/usr/bin/sort", args);
        exit(1); //encerrar o processo filho D e liberar os seus recursos
    }

    waitpid(pidD, NULL, 0);
}
